<?php

namespace Woo_MP\Payment_Gateways\Stripe;

use Woo_MP\Detailed_Exception;
use Woo_MP\Payment_Gateways\Stripe\API\Exceptions\Stripe_Exception;

defined( 'ABSPATH' ) || die;

/**
 * Process a payment with Stripe.
 */
class Payment_Processor extends Transaction_Processor {

    /**
     * Process a payment.
     *
     * @param array $params The following payment parameters are required:
     *
     * [
     *     'token'       => '', // Token representing the payment source.
     *
     *     // The following parameters are provided by {@see \Woo_MP\Payment_Processor::process()}.
     *     'order'       => null,
     *     'capture'     => false,
     *     'description' => ''
     * ]
     *
     * @see \Woo_MP\Payment_Processor::process() For parameters required for all payment processors.
     *
     * @return array Result:
     *
     * [
     *     'trans_id'        => '',
     *     'held_for_review' => false
     * ]
     *
     * @throws Detailed_Exception For detailed errors.
     */
    public function process( $params ) {
        $amount = $this->to_smallest_unit( $params['amount'], $params['currency'] );

        $moto_enabled = get_option( 'woo_mp_stripe_moto_enabled' ) === 'yes';

        $request = [
            'amount'              => $amount,
            'currency'            => $params['currency'],
            'payment_method_data' => [
                'type' => 'card',
                'card' => [ 'token' => $params['token'] ],
            ],
            'confirm'             => true,
            'capture_method'      => $params['capture'] ? 'automatic' : 'manual',
            'description'         => $params['description'],
            'metadata'            => [ 'Order Number' => $params['order']->get_order_number() ],
        ];

        if ( $moto_enabled ) {
            $request['payment_method_options'] = [ 'card' => [ 'moto' => true ] ];
        } else {
            $request['off_session'] = true;
        }

        if ( get_option( 'woo_mp_stripe_include_name_and_email', 'yes' ) === 'yes' ) {
            $name  = $params['order']->get_billing_first_name() . ' ' . $params['order']->get_billing_last_name();
            $email = $params['order']->get_billing_email();

            $request['metadata']['Customer Name']  = $name;
            $request['metadata']['Customer Email'] = $email;
        }

        $request = apply_filters( 'woo_mp_stripe_charge_request', $request, $params['order']->get_core_order() );

        try {
            $payment_intent = $this->request( 'POST', 'payment_intents', $request );
        } catch ( Detailed_Exception $e ) {
            $message      = '';
            $code         = $e->getCode();
            $stripe_error = $e->getPrevious() instanceof Stripe_Exception ? $e->getPrevious()->get_error() : null;

            if ( $code === 'authentication_required' ) {
                if ( $moto_enabled ) {
                    $message =
                        'Strong Customer Authentication is required for this payment. ' .
                        "The cardholder's bank has rejected the Mail Order / Telephone Order (MOTO) exemption. " .
                        'This means that the customer will have to visit your website to pay for the order.';
                } else {
                    $message =
                        'Strong Customer Authentication is required for this payment. ' .
                        'This means that the customer will have to visit your website to pay for the order. ' .
                        'You may be able to prevent this error for future payments by using the ' .
                        'Mail Order / Telephone Order (MOTO) exemption. You will need to contact Stripe ' .
                        'to get this feature enabled for your account. Once the feature is enabled, ' .
                        'you will be able to use the Mark Payments as MOTO setting.';
                }
            }

            if (
                $code === 'parameter_unknown' &&
                isset( $stripe_error->param ) &&
                $stripe_error->param === 'payment_method_options[card][moto]'
            ) {
                $message =
                    'The Mark Payments as MOTO setting is enabled, ' .
                    'however your Stripe account does not have the MOTO feature enabled. ' .
                    'You will need to contact Stripe to get this feature enabled for your account.';
            }

            if ( $message ) {
                throw new Detailed_Exception( $message, $code, $e->get_data(), $e );
            }

            throw $e;
        }

        $charge = $payment_intent->charges->data[0];

        return [
            'trans_id'        => $charge->id,
            'held_for_review' => $charge->outcome->type === 'manual_review',
        ];
    }

}
